@@ -1,6 +1,6 @@
 module Openwsman
   class XmlNode
-    def method_missing method, *args
+    def method_missing method, *args # :nodoc:
       find(nil, method.to_s)
     end
   end
