@@ -1,10 +1,8 @@
 # get.rb
 
-$:.unshift "../../../build/bindings/ruby"
-$:.unshift "../.libs"
-
 require 'test/unit'
 require 'rexml/document'
+require File.join(File.dirname(__FILE__),'_loadpath')
 require 'openwsman'
 require '_client'
 
@@ -22,7 +20,7 @@ class WsmanTest < Test::Unit::TestCase
     result = client.get( options, uri )
     assert result
 
-    puts result
+    puts result.to_xml
   end
 end
 
