@@ -1,26 +1,25 @@
 # put.rb
 
-$:.unshift "../../../build/bindings/ruby"
-$:.unshift "../.libs"
-
 require 'test/unit'
 require 'rexml/document'
+require File.join(File.dirname(__FILE__),'_loadpath')
 require 'openwsman'
 require '_client'
 
 class WsmanTest < Test::Unit::TestCase
   def test_client
+#    Openwsman.debug = -1
     client = Client.open
     assert client
     options = Openwsman::ClientOptions.new
     assert options
-    uri = "http://schema.omc-project.org/wbem/wscim/1/cim-schema/2/OMC_TimeZoneSettingData"
+#    options.set_dump_request
+    uri = "http://schemas.dmtf.org/wbem/wscim/1/cim-schema/2/CIM_TimeZoneSettingData"
     data = Openwsman::XmlDoc.new("data")
-    data_s = data.to_s
+    data_s = data.to_xml
     result = client.put( options, uri, data_s, data_s.size, client.encoding )
-    assert result
 
-    doc = REXML::Document.new result.to_s
+    doc = REXML::Document.new result.to_xml
     assert doc
 #    doc.write( $stdout, 0 )
   end
