@@ -39,7 +39,8 @@
 extern "C" {
 #endif				/* __cplusplus */
 
-#include "u/libu.h"
+#include <stdio.h>
+
 #include "wsman-types.h"
 
 
@@ -94,11 +95,17 @@ typedef int (*WsXmlNsEnumCallback) (WsXmlNodeH, WsXmlNsH, void *);
 	// Dumping
 void ws_xml_dump_node_tree(FILE * f, WsXmlNodeH node);
 
+/* indented, using doc encoding */
 void ws_xml_dump_memory_node_tree(WsXmlNodeH node, char **buf,
 				  int *ptrSize);
 
+/* indented, using given encoding (default: utf-8) */
+void ws_xml_dump_memory_node_tree_enc(WsXmlNodeH node, char **buf,
+                                      int *ptrSize, const char *encoding);
+
 void ws_xml_dump_doc(FILE * f, WsXmlDocH doc);
 
+/* non-indented, using given encoding */
 void ws_xml_dump_memory_enc(WsXmlDocH doc, char **buf, int *ptrSize,
 			    const char *encoding);
 
@@ -132,6 +139,9 @@ WsXmlNodeH ws_xml_get_node_parent(WsXmlNodeH node);
 
 int ws_xml_get_child_count(WsXmlNodeH parent);
 
+int ws_xml_get_child_count_by_qname(WsXmlNodeH parent,
+				    const char *nsUri, const char *name);
+
 WsXmlNodeH ws_xml_get_child(WsXmlNodeH parent, int index,
 			    const char *nsUri, const char *localName);
 
