@@ -36,6 +36,7 @@
 #include <wsman_config.h>
 #endif
 
+#include "u/libu.h"
 #include "wsman-xml-api.h"
 #include "wsman-soap.h"
 #include "wsman-dispatcher.h"
@@ -173,6 +174,7 @@ wsmc_build_envelope(WsSerializerContextH serctx,
 				options->flags & FLAG_MUND_MAX_ESIZE);
 	}
 	if (options->fragment) {
+          /* FIXME: mu set but unused */
 		int mu = 0;
 		if ((options->flags & FLAG_MUND_FRAGMENT) ==
 				FLAG_MUND_FRAGMENT)
@@ -190,9 +192,16 @@ wsmc_build_envelope(WsSerializerContextH serctx,
 		wsmc_add_selector_from_options(doc, options);
 
 		if (options->cim_ns) {
-			wsman_add_selector(header,
-					CIM_NAMESPACE_SELECTOR, options->cim_ns);
-		}
+                  /* don't add CIM_NAMESPACE_SELECTOR twice */
+                  if (options->selectors && hash_count(options->selectors) > 0) {
+                    if (!hash_lookup(options->selectors, CIM_NAMESPACE_SELECTOR)) {
+                      wsman_add_selector(header, CIM_NAMESPACE_SELECTOR, options->cim_ns);
+                    }
+                  }
+                  else {
+                    wsman_add_selector(header, CIM_NAMESPACE_SELECTOR, options->cim_ns);
+                  }
+                }
 	}
 	return doc;
 }
@@ -312,6 +321,9 @@ wsmc_options_init(void)
 void
 wsmc_options_destroy(client_opt_t * op)
 {
+	if (op->options) {
+		hash_free(op->options);
+	}
 	if (op->selectors) {
 		hash_free(op->selectors);
 	}
@@ -335,6 +347,13 @@ wsmc_set_action_option(client_opt_t * options, unsigned int flag)
 }
 
 
+unsigned int
+wsmc_get_action_option(client_opt_t * options)
+{
+	return options->flags;
+}
+
+
 void
 wsmc_clear_action_option(client_opt_t * options, unsigned int flag)
 {
@@ -360,6 +379,27 @@ wsmc_add_property(client_opt_t * options,
 	}
 }
 
+/*
+ * add key/value pair to WSM_OPTION_SET
+ *
+ */
+void
+wsmc_add_option(client_opt_t * options,
+		const char *key,
+		const char *value)
+{
+	if (options->options == NULL)
+		options->options = hash_create(HASHCOUNT_T_MAX, 0, 0);
+	if (!hash_lookup(options->options, key)) {
+		if (!hash_alloc_insert(options->options,
+					(char *)key, (char *)value)) {
+			error( "hash_alloc_insert failed");
+		}
+	} else {
+		error( "duplicate not added to hash");
+	}
+}
+
 void
 wsmc_add_selector(client_opt_t * options,
 		const char *key,
@@ -416,21 +456,48 @@ wsmc_add_selector_from_options(WsXmlDocH doc, client_opt_t *options)
 	while ((hn = hash_scan_next(&hs))) {
 		wsman_add_selector(header,
 				(char *) hnode_getkey(hn), (char *) hnode_get(hn));
-		debug("key = %s value=%s",
+		debug("key=%s value=%s",
 				(char *) hnode_getkey(hn), (char *) hnode_get(hn));
 	}
 }
 
+/* Err: should be wsmc_set_selectors_from_uri() */
 void
 wsmc_set_options_from_uri(const char *resource_uri, client_opt_t * options)
 {
-	options->selectors = get_selectors_from_uri(resource_uri);
+  if (options->selectors) {
+    hash_free_nodes(options->selectors);
+    hash_destroy(options->selectors);
+  }
+  options->selectors = get_selectors_from_uri(resource_uri);
 }
 
 void
 wsmc_set_delivery_uri(const char *delivery_uri, client_opt_t * options)
 {
-	options->delivery_uri = u_strdup(delivery_uri);
+  u_free(options->delivery_uri);
+  options->delivery_uri = delivery_uri ? u_strdup(delivery_uri) : NULL;
+}
+
+void
+wsmc_set_cim_ns(const char *ns, client_opt_t * options)
+{
+  u_free(options->cim_ns);
+  options->cim_ns = ns ? u_strdup(ns) : NULL;
+}
+
+void
+wsmc_set_fragment(const char *fragment, client_opt_t * options)
+{
+  u_free(options->fragment);
+  options->fragment = fragment ? u_strdup(fragment) : NULL;
+}
+
+void
+wsmc_set_reference(const char *reference, client_opt_t * options)
+{
+  u_free(options->reference);
+  options->reference = reference ? u_strdup(reference) : NULL;
 }
 
 void
@@ -793,9 +860,10 @@ wsmc_create_request(WsManClient * cl, const char *resource_uri,
 		request = ws_xml_create_envelope();
 	} else {
 		if (method) {
+		        /* WSA_ACTION is a URI */
 			if (strchr(method, '/'))
 				_action = u_strdup(method);
-			else
+			else /* 'invoke' action */
 				_action = wsman_make_action((char *)resource_uri, method);
 		} else {
 			_action = wsmc_create_action_str(action);
@@ -815,9 +883,13 @@ wsmc_create_request(WsManClient * cl, const char *resource_uri,
 	if (!body  || !header )
 		return NULL;
 	/*
-	 * flags to be passed as <w:OptionSet ...> <w:Option Name="..." ...> > */
-	if (options && (options->flags & (FLAG_CIM_EXTENSIONS|FLAG_EXCLUDE_NIL_PROPS))) {
-		WsXmlNodeH opset = ws_xml_add_child(header,
+	 * flags to be passed as <w:OptionSet ...> <w:Option Name="..." ...> >
+	 */
+	if (options) {
+	  WsXmlNodeH opset = NULL;
+	  if (options->flags & (FLAG_CIM_EXTENSIONS|FLAG_EXCLUDE_NIL_PROPS)) {
+	    /* need WSM_OPTION_SET */
+		opset = ws_xml_add_child(header,
 				XML_NS_WS_MAN, WSM_OPTION_SET, NULL);
 		if ((options->flags & FLAG_CIM_EXTENSIONS) == FLAG_CIM_EXTENSIONS) {
 			WsXmlNodeH op = ws_xml_add_child(opset,
@@ -830,6 +902,22 @@ wsmc_create_request(WsManClient * cl, const char *resource_uri,
 				XML_NS_OPENWSMAN, WSM_OPTION, NULL);
 			ws_xml_add_node_attr(op, NULL, WSM_NAME, WSMB_EXCLUDE_NIL_PROPS);
 		}
+	  }
+	  if (options->options && hash_count(options->options) > 0) {
+	    hnode_t *hn;
+	    hscan_t hs;
+	    hash_scan_begin(&hs, options->options);
+	    if (!opset) {
+	      opset = ws_xml_add_child(header,
+				       XML_NS_WS_MAN, WSM_OPTION_SET, NULL);
+	    }
+	    while ((hn = hash_scan_next(&hs))) {
+	      WsXmlNodeH op = ws_xml_add_child(opset,
+					       XML_NS_WS_MAN, WSM_OPTION, NULL);
+	      ws_xml_add_node_attr(op, NULL, WSM_NAME, (char *) hnode_getkey(hn));
+	      ws_xml_set_node_text(op, (char *) hnode_get(hn));
+	    }
+	  }
 	}
 
 
@@ -897,13 +985,14 @@ wsmc_create_request(WsManClient * cl, const char *resource_uri,
 	if (action == WSMAN_ACTION_PULL || action == WSMAN_ACTION_ENUMERATION) {
 		if (options->max_elements > 0 ) {
 			node = ws_xml_get_child(body, 0, NULL, NULL);
-			if ((action == WSMAN_ACTION_ENUMERATION)
-			    && (options->flags & FLAG_ENUMERATION_OPTIMIZATION) ==
+			if (action == WSMAN_ACTION_ENUMERATION) {
+				if ((options->flags & FLAG_ENUMERATION_OPTIMIZATION) ==
 					FLAG_ENUMERATION_OPTIMIZATION ) {
-				/* wsman:MaxElements is for Enumerate */
-				ws_xml_add_child_format(node, XML_NS_WS_MAN,
-						WSENUM_MAX_ELEMENTS, "%d", options->max_elements);
-			} else {
+						/* wsman:MaxElements is for Enumerate */
+						ws_xml_add_child_format(node, XML_NS_WS_MAN,
+									WSENUM_MAX_ELEMENTS, "%d", options->max_elements);
+				}
+		         } else {
 				/* wsen:MaxElements is for Pull */
 				ws_xml_add_child_format(node, XML_NS_ENUMERATION,
 						WSENUM_MAX_ELEMENTS, "%d", options->max_elements);
@@ -1228,10 +1317,12 @@ wsmc_action_invoke(WsManClient * cl,
                             (char *) hnode_get(hn));
                 }
             }
-        } else {
+        } else if (!strchr(method, '/')) { /* non-custom method without parameters */
             ws_xml_add_empty_child_format(body,
                     (char *)resource_uri, "%s_INPUT", method);
-        }
+        } else {
+	  /* empty body */
+	}
 	if ((options->flags & FLAG_DUMP_REQUEST) == FLAG_DUMP_REQUEST) {
 		ws_xml_dump_node_tree(cl->dumpfile, ws_xml_get_doc_root(request));
 	}
@@ -1354,7 +1445,7 @@ wsmc_action_enumerate_and_pull(WsManClient * cl,
 		void *callback_data)
 {
 	WsXmlDocH doc;
-	char *enumContext;
+	char *enumContext = NULL;
 	WsXmlDocH enum_response = wsmc_action_enumerate(cl,
 			resource_uri, options, filter);
 
@@ -1376,14 +1467,17 @@ wsmc_action_enumerate_and_pull(WsManClient * cl,
 		doc = wsmc_action_pull(cl, resource_uri, options, filter, enumContext);
 
 		if (rc != 200 && rc != 400 && rc != 500) {
+			wsmc_free_enum_context(enumContext);
 			return 0;
 		}
 		callback(cl, doc, callback_data);
+		wsmc_free_enum_context(enumContext);
 		enumContext = wsmc_get_enum_context(doc);
 		if (doc) {
 			ws_xml_destroy_doc(doc);
 		}
 	}
+	wsmc_free_enum_context(enumContext);
 	return 1;
 }
 
@@ -1665,14 +1759,16 @@ wsmc_create_from_uri(const char* endpoint)
 int
 wsmc_set_encoding(WsManClient *cl, const char *encoding)
 {
-	cl->content_encoding = u_strdup(encoding);
+	u_free(cl->content_encoding);
+	cl->content_encoding = encoding ? u_strdup(encoding) : NULL;
 	return 0;
 }
 
 int
 wsmc_set_namespace(WsManClient *cl, const char *ns)
 {
-	cl->cim_ns = u_strdup(ns);
+	u_free(cl->cim_ns);
+	cl->cim_ns = ns ? u_strdup(ns) : NULL;
 	return 0;
 }
 
@@ -1794,7 +1890,7 @@ wsmc_create(const char *hostname,
 	wsc->session_handle = 0;
 #endif
 	wsc->data.endpoint = u_strdup_printf("%s://%s:%d%s",
-			scheme, hostname, port, path);
+			wsc->data.scheme, wsc->data.hostname, wsc->data.port, wsc->data.path);
 	debug("Endpoint: %s", wsc->data.endpoint);
 	wsc->authentication.verify_host = 1; //verify CN in server certicates by default
 	wsc->authentication.verify_peer = 1; //validate server certificates by default
@@ -1813,7 +1909,12 @@ wsmc_create(const char *hostname,
 void
 wsmc_release(WsManClient * cl)
 {
-
+#ifndef _WIN32
+	if (cl->client_config_file) {
+		u_free(cl->client_config_file);
+		cl->client_config_file = NULL;
+	}
+#endif
 	if (cl->data.scheme) {
 		u_free(cl->data.scheme);
 		cl->data.scheme = NULL;
@@ -1864,7 +1965,28 @@ wsmc_release(WsManClient * cl)
 		u_free(cl->authentication.crl_file);
 		cl->authentication.crl_file = NULL;
 	}
-	 
+	if (cl->authentication.method != NULL) {
+		u_free(cl->authentication.method);
+		cl->authentication.method = NULL;
+	}
+        if (cl->authentication.cainfo != NULL) {	 
+          u_free(cl->authentication.cainfo);
+          cl->authentication.cainfo = NULL;
+        }
+        if (cl->authentication.caoid != NULL) {
+          u_free(cl->authentication.caoid);
+          cl->authentication.caoid = NULL;
+        }
+        if (cl->proxy_data.proxy_username != NULL) {
+          u_free(cl->proxy_data.proxy_username);
+          cl->proxy_data.proxy_username = NULL;
+        }
+        if (cl->proxy_data.proxy_password != NULL) {
+          u_free(cl->proxy_data.proxy_password);
+          cl->proxy_data.proxy_password = NULL;
+        }
+        pthread_mutex_destroy(&cl->mutex);
+
 	wsman_transport_close_transport(cl);
 
 	u_free(cl);
