@@ -105,8 +105,8 @@ reauthenticate(WsManClient *cl,
 		goto REQUEST_PASSWORD;
 	}
 
-	debug("Client does not support authentication type "
-			" acceptable by server\n");
+	debug("Client does not support authentication type 0x%04x"
+			" acceptable by server\n", auth_avail);
 	return 0;
 
 
@@ -118,8 +118,22 @@ REQUEST_PASSWORD:
 		return choosen_auth;
 	}
 
-	if ( cl->authentication.auth_request_func )
+	if (cl->authentication.auth_request_func) {
+		debug("Invoking Auth request callback");
+
+		/* free the username and password as these are OUT params to the auth_request_func
+		 * which the caller is expected to set
+		 */
+		if (*username) {
+			u_free(*username);
+			*username = NULL;
+		}
+		if (*password) {
+			u_free(*password);
+			*password = NULL;
+		}
 		cl->authentication.auth_request_func(cl, ws_auth, username, password);
+	}
 	else
 		return 0;
 
@@ -168,6 +182,8 @@ convert_to_last_error(CURLcode r)
 		return WS_LASTERR_TOO_MANY_REDIRECTS;
 	case CURLE_SSL_CONNECT_ERROR:
 		return WS_LASTERR_SSL_CONNECT_ERROR;
+        case CURLE_BAD_FUNCTION_ARGUMENT:
+                return WS_LASTERR_CURL_BAD_FUNCTION_ARG;
 	case CURLE_SSL_PEER_CERTIFICATE:
 		return WS_LASTERR_SSL_PEER_CERTIFICATE;
 	case CURLE_SSL_ENGINE_NOTFOUND:
@@ -218,6 +234,7 @@ write_handler( void *ptr, size_t size, size_t nmemb, void *data)
 	debug("write_handler: recieved %d bytes, all = %d\n", len, u_buf_len(buf));
 	return len;
 }
+
 #ifdef ENABLE_EVENTING_SUPPORT
 static int ssl_certificate_thumbprint_verify_callback(X509_STORE_CTX *ctx, void *arg)
 {
@@ -256,20 +273,19 @@ init_curl_transport(WsManClient *cl)
 	curl = curl_easy_init();
 	if (curl == NULL) {
 		r = CURLE_FAILED_INIT;
-		curl_global_cleanup();
 		debug("Could not init easy curl");
 		goto DONE;
 	}
 	debug("cl->authentication.verify_peer: %d", cl->authentication.verify_peer );
 	// verify peer
-	r = curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, cl->authentication.verify_peer);
+	r = curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, wsman_transport_get_verify_peer(cl)?1:0);
 	if (r != 0) {
 		curl_err("curl_easy_setopt(CURLOPT_SSL_VERIFYPEER) failed");
 		goto DONE;
 	}
 
 	// verify host
-	r = curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, cl->authentication.verify_host);
+	r = curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, wsman_transport_get_verify_host(cl)?2:0);
 	if (r != 0) {
 		curl_err("curl_easy_setopt(CURLOPT_SSL_VERIFYHOST) failed");
 		goto DONE;
@@ -333,7 +349,7 @@ init_curl_transport(WsManClient *cl)
 #ifdef ENABLE_EVENTING_SUPPORT
 /*  Bug in e.g. Fedora: [ curl-Bugs-1924441 ] SSL callback option with NSS-linked libcurl */
 #ifndef NO_SSL_CALLBACK
-	else if (cl->authentication.certificatethumbprint && 0 != cl->authentication.verify_peer) {
+	else if (strlen((char *)cl->authentication.certificatethumbprint) > 0 && 0 != cl->authentication.verify_peer) {
 		r = curl_easy_setopt(curl, CURLOPT_SSL_CTX_FUNCTION, sslctxfun);
 		if(r != 0) {
 			curl_err("Could not curl_easy_setopt(curl, CURLOPT_SSL_CTX_FUNCTION)");
@@ -398,12 +414,11 @@ wsmc_handler( WsManClient *cl,
 	}
 	if (cl->transport == NULL) {
 		cl->transport = init_curl_transport(cl);
+                if (cl->transport == NULL) {
+                        return;
+                }
 	}
 	curl = (CURL *)cl->transport;
-	if (curl == NULL) {
-		cl->last_error = WS_LASTERR_FAILED_INIT;
-		return;
-	}
 
 	r = curl_easy_setopt(curl, CURLOPT_URL, cl->data.endpoint);
 	if (r != CURLE_OK) {
@@ -558,17 +573,6 @@ wsmc_handler( WsManClient *cl,
 			goto DONE;
 		}
 
-		if (cl->data.auth_set) {
-			if (cl->data.user) {
-				u_free(cl->data.user);
-				cl->data.user = NULL;
-			}
-			if (cl->data.pwd) {
-				u_free(cl->data.pwd);
-				cl->data.pwd = NULL;
-			}
-		}
-
 		cl->data.auth_set = reauthenticate(cl, cl->data.auth_set, auth_avail,
                         &cl->data.user, &cl->data.pwd);
                 u_buf_clear(response);
