@@ -54,6 +54,7 @@
 
 
 #define FAULT_XPATH_EXPR  "/s:Envelope/s:Body/s:Fault/s:Code/s:Value"
+#define FAULT_SUBCODE_VALUE_XPATH "/s:Envelope/s:Body/s:Fault/s:Code/s:Subcode/s:Value"
 
 WsmanFaultDetailTable fault_detail_table[] =
 {
@@ -141,10 +142,10 @@ WsmanFaultCodeTable fault_code_table[] =
 		XML_NS_SOAP_1_2,
 		FAULT_MUSTUNDERSTAND_CODE,
 		"",
-		"The WS-Management service cannot process a SOAP header in the request that" \
-		"is marked as mustUnderstand by the client.  This could be caused by the use" \
-		"of a version of the protocol which is not supported, or may be an" \
-		"incompatibility  between the client and server implementations."
+		"The WS-Management service cannot process a SOAP header in the request that " \
+		"is marked as mustUnderstand by the client. This could be caused by the use " \
+		"of a version of the protocol which is not supported, or may be an " \
+		"incompatibility between the client and server implementations."
 	},
 	{
 		WSA_ENDPOINT_UNAVAILABLE,
@@ -602,5 +603,27 @@ wsman_fault_occured(WsmanMessage *msg)
 }
 
 
+void
+wsman_get_fault_status_from_doc (WsXmlDocH doc, WsmanStatus *status)
+{
+  int i;
+  char *subcode_value=ws_xml_get_xpath_value(doc, FAULT_SUBCODE_VALUE_XPATH);
+  char *subcode_value_msg =calloc(1,strlen(subcode_value));
+  char *start_pos = strchr(subcode_value,':');
+  strcpy(subcode_value_msg, start_pos+1); 
+  if (strlen(subcode_value)== 0 ) return ;
+ 
+  int nfaults = sizeof (fault_code_table) / sizeof (fault_code_table[0]);
+  for (i = 0; i < nfaults; i++) {
+    if (strcmp (subcode_value_msg , fault_code_table[i].subCode) == 0) {
+      status->fault_code = fault_code_table[i].fault_code;
+      //some default values
+      status->fault_detail_code = 0;
+      status->fault_msg='\0';
+      return;
+    }
 
+  }
+  return;
+}
 
