@@ -501,19 +501,32 @@ wsman_create_fault_envelope(WsXmlDocH rqstDoc,
 	return doc;
 }
 
+/*
+ * Interpret query as XPath
+ * 
+ */
+
 static int interpretxpath(char **xpath)
 {
-	return 1;
+	return 0;
 }
 
+/*
+ * Parse enumeration request
+ * @return: 0 for error, set WsmanStatus accordingly
+ * 
+ */
 int wsman_parse_enum_request(WsContextH cntx,
-		WsEnumerateInfo * enumInfo)
+		WsEnumerateInfo * enumInfo, WsmanStatus *status)
 {
 	filter_t *filter = NULL;
 	WsXmlNodeH node;
 	WsXmlDocH doc = cntx->indoc;
-	if (!doc)
+	if (!doc) {
+                status->fault_code = WSMAN_INVALID_PARAMETER;
+                status->fault_detail_code = WSMAN_DETAIL_INVALID_VALUE;
 		return 0;
+        }
 
 	node = ws_xml_get_soap_body(doc);
 	if (node && (node = ws_xml_get_child(node, 0,
@@ -580,7 +593,7 @@ int wsman_parse_enum_request(WsContextH cntx,
 				else
 					enumInfo->flags |= WSMAN_ENUMINFO_REF;
 			}
-			else if(strcmp(filter->dialect, WSM_CQL_FILTER_DIALECT) ==0)
+			else if(strcmp(filter->dialect, WSM_CQL_FILTER_DIALECT) == 0)
 				enumInfo->flags |= WSMAN_ENUMINFO_CQL;
 			else if(strcmp(filter->dialect, WSM_WQL_FILTER_DIALECT) == 0)
 				enumInfo->flags |= WSMAN_ENUMINFO_WQL;
@@ -588,9 +601,12 @@ int wsman_parse_enum_request(WsContextH cntx,
 				enumInfo->flags |= WSMAN_ENUMINFO_SELECTOR;
 			else {
 				if(interpretxpath(&filter->query))
-					enumInfo->flags |= WSMAN_ENUMINFO_WQL;
-				else
+					enumInfo->flags |= WSMAN_ENUMINFO_XPATH;
+				else {
+                                        status->fault_code = WSEN_CANNOT_PROCESS_FILTER;
+                                        status->fault_detail_code = WSMAN_DETAIL_NOT_SUPPORTED;
 					return 0;
+                                }
 
 			}
 		}
@@ -789,7 +805,7 @@ int wsman_get_max_elements(WsContextH cntx, WsXmlDocH doc)
 
 		if (node && (node = ws_xml_get_child(node, 0, XML_NS_ENUMERATION,
 					 WSENUM_PULL))) {
-			node = ws_xml_get_child(node, 0, XML_NS_WS_MAN,
+			node = ws_xml_get_child(node, 0, XML_NS_ENUMERATION,
 					     WSENUM_MAX_ELEMENTS);
 			if (node) {
 				char *text = ws_xml_get_node_text(node);
@@ -980,9 +996,9 @@ hash_t *
 wsman_get_method_args(WsContextH cntx, const char *resource_uri)
 {
 	char *input = NULL;
+	WsXmlDocH doc = cntx->indoc;
 	hash_t *h = hash_create(HASHCOUNT_T_MAX, 0, 0);
 	hash_set_allocator(h, NULL, wsman_free_method_hnode, NULL);
-	WsXmlDocH doc = cntx->indoc;
 	if (doc) {
 		WsXmlNodeH in_node;
 		WsXmlNodeH body = ws_xml_get_soap_body(doc);
