@@ -37,6 +37,7 @@
 #endif
 
 
+#include "u/libu.h"
 #include "wsman-xml-api.h"
 #include "wsman-client-api.h"
 #include "wsman-soap.h"
@@ -79,6 +80,7 @@ static WsXmlNsData     g_wsNsData[] =
 	{XML_NS_CIM_BINDING, "wsmb"},
 	{XML_NS_OPENWSMAN, "owsman"},
 	{XML_NS_TRANSFER, "wxf"},
+	{XML_NS_XML_NAMESPACES,"xml"},
 	{NULL, NULL}
 };
 
@@ -362,7 +364,7 @@ int ws_xml_utf8_strlen(char *buf)
 }
 
 /**
- * Dump XML docuemnt contents into a Text buffer
+ * Dump XML document contents into a Text buffer
  * @param doc XML document
  * @param buf The target buffer
  * @param ptrSize the size of the buffer
@@ -798,6 +800,39 @@ int ws_xml_is_node_qname(WsXmlNodeH node, const char *nsUri,
 }
 
 
+/**
+ * Count number of XML node children with same qualified name
+ * (used to represent array elements)
+ * @param parent XML node
+ * @param nsUri Namespace URI
+ * @param name children name to look for
+ * @return Returns number of children of parent with this name
+ * @brief Identical to ws_xml_get_child_count() if nsUri==NULL and name==NULL
+ */
+int 
+ws_xml_get_child_count_by_qname(WsXmlNodeH parent, 
+      const char *nsUri, const char *name)
+{
+	WsXmlNodeH node;
+	int count;
+
+	if (!parent)
+	        return 0;
+	if (nsUri == NULL && name == NULL) {
+		return ws_xml_get_child_count(parent);
+	}
+	node = xml_parser_get_first_child(parent);
+	count = 0;
+	while (node != NULL) {
+		if (ws_xml_is_node_qname(node, nsUri, name)) {
+			count++;
+		}
+		node = xml_parser_get_next_child(node);
+	}
+	return count;
+}
+
+
 WsXmlDocH ws_xml_create_soap_envelope(void)
 {
 	return ws_xml_create_envelope();
@@ -1392,6 +1427,14 @@ void ws_xml_dump_memory_node_tree(WsXmlNodeH node, char **buf,
 	return;
 }
 
+void ws_xml_dump_memory_node_tree_enc(WsXmlNodeH node, char **buf,
+				  int *ptrSize, const char *encoding)
+{
+	WsXmlDocH doc = xml_parser_get_doc(node);
+	xml_parser_doc_dump_memory_enc(doc, buf, ptrSize, encoding);
+	return;
+}
+
 void ws_xml_dump_doc(FILE * f, WsXmlDocH doc)
 {
 	xml_parser_doc_dump(f, doc);
