@@ -180,10 +180,12 @@ verify_class_namespace(CimClientInfo *client)
 	        return 1;
 	}
   
-        /* Ok if class contains CIM, uri contains XML_NS_CIM_CLASS, and method is not 'Create' */
+        /* Ok if class schema is CIM, uri starts with XML_NS_CIM_CLASS
+         * and method is not 'Create' (CIM_ is abstract, cannot be created)
+         */
 	if (client->requested_class && client->resource_uri && client->method
-            && (strstr(client->requested_class, "CIM") != NULL )
-	    && (strstr(client->resource_uri , XML_NS_CIM_CLASS) != NULL )
+            && (strncmp(client->requested_class, "CIM_", 4) == 0 )
+	    && (strstr(client->resource_uri , XML_NS_CIM_CLASS) == client->resource_uri )
 	    && (strcmp(client->method, TRANSFER_CREATE) != 0)) {
 		return 1;
 	}
@@ -439,7 +441,10 @@ CimResource_Enumerate_EP( WsContextH cntx,
 		goto cleanup;
 	}
 
-	wsman_parse_enum_request(cntx, enumInfo);
+	if (!wsman_parse_enum_request(cntx, enumInfo, status)) {
+          retval = 1;
+          goto cleanup;
+        }
 	cim_enum_instances(cimclient, enumInfo, status);
 
 	if (status && status->fault_code != 0) {
@@ -461,6 +466,12 @@ CimResource_Enumerate_EP( WsContextH cntx,
 		}
 	}
 cleanup:
+	/* the cimclient is stored inside the enumInfo and kept in-memory
+	 * during the Pull_EP requests until the final Release_EP call.
+	 * 
+	 * Only destroy it here in case of error
+	 * 
+	 */
 	if (retval && cimclient) {
 		CimResource_destroy(cimclient);
 	}
@@ -765,7 +776,7 @@ CimResource_Subscribe_EP(WsContextH cntx,
 	subsInfo->vendor_namespaces = cimclient->namespaces;
 	subsInfo->cim_namespace = u_strdup(cimclient->cim_namespace);
 	if(subsInfo->flags & WSMAN_SUBSCRIPTION_SELECTORSET) { //Subscribe to an Indication filter instance
-		indicationfilter= cim_get_objectpath_from_selectors(cimclient, cntx, status);
+		indicationfilter = cim_get_indicationfilter_objectpath_from_selectors(cimclient, cntx, status);
 		if(indicationfilter)
 			subsInfo->existingfilterOP = CMClone(indicationfilter, NULL);
 		else {
