@@ -45,11 +45,11 @@
 #include "u/libu.h"
 
 
-#include "wsman-xml-api.h"
+#include "wsman-xml.h"
+#include "wsman-xml-binding.h"
 #include "wsman-client-api.h"
 #include "wsman-soap.h"
 #include "wsman-soap-envelope.h"
-#include "wsman-xml.h"
 #include "wsman-epr.h"
 
 #include "sfcc-interface.h"
@@ -66,6 +66,17 @@ typedef struct _sfcc_enumcontext {
 	CMPIEnumeration *ecEnumeration;
 } sfcc_enumcontext;
 
+static int cim_getEprObjAt(CimClientInfo * client, WsEnumerateInfo * enumInfo,
+		    WsXmlNodeH itemsNode);
+
+static int cim_getEprAt(CimClientInfo * client, WsEnumerateInfo * enumInfo,
+		 WsXmlNodeH itemsNode);
+
+static int cim_getElementAt(CimClientInfo * client, WsEnumerateInfo * enumInfo,
+		     WsXmlNodeH itemsNode);
+
+
+
 static char *
 cim_find_namespace_for_class(CimClientInfo * client,
 		WsEnumerateInfo * enumInfo,
@@ -206,7 +217,7 @@ path2xml(CimClientInfo * client,
 		WsXmlNodeH s = NULL;
 		CMPIData data = objectpath->ft->getKeyAt(objectpath, i,
 				&keyname, NULL);
-		cv = (char *) value2Chars(data.  type, &data.  value);
+		cv = (char *) value2Chars(data.  type, &data.value);
 		s = ws_xml_add_child(wsman_selector_set, XML_NS_WS_MAN,
 				WSM_SELECTOR, cv );
 		ws_xml_add_node_attr(s, NULL, "Name", CMGetCharPtr(keyname));
@@ -239,13 +250,28 @@ path2xml(CimClientInfo * client,
  * O: data.value = type-correct value
  */
 static void
-xml2data(CMPIData *data, char *value)
+xml2data(CMPIData *data, char *value, char *name, 
+   WsXmlNodeH resource, const char *resource_uri)
 {
 	CMPIType type = data->type;
 
 	if (type & CMPI_ARRAY) {
-		debug("*** xml2data: Array unsupported");
-	        data->value.array = newCMPIArray(0, type, NULL);
+		WsXmlNodeH child;
+		CMPIData adata;
+		int count, ii;
+
+		adata.type = (type ^ CMPI_ARRAY);
+		count = ws_xml_get_child_count_by_qname(resource, resource_uri, name);
+		data->value.array = newCMPIArray(count, adata.type, NULL);
+		for (ii = 0; ii < count; ii++) {
+			child = ws_xml_get_child(resource, ii, resource_uri, name);
+			if (child) {
+				if ((value = ws_xml_get_node_text(child))) {
+					xml2data(&adata, value, name, resource, resource_uri);
+					CMSetArrayElementAt(data->value.array, ii, &(adata.value), adata.type);
+				}
+			}
+		}
 	} else {
 		switch (type) {
 		case CMPI_instance:
@@ -322,17 +348,18 @@ xml2objectpath(CMPIObjectPath * objectpath,
 	         CMPIData *data, char *name, char *value)
 {
   debug("xml2objectpath([0x%04x]%s:%s)", data->type, name, value);
-        xml2data(data, value);
+        xml2data(data, value, NULL, NULL, NULL);
         CMAddKey(objectpath, name, &(data->value), data->type);
 }
 
 
 void
 xml2property(CMPIInstance * instance,
-		CMPIData *data, char *name, char *value)
+		CMPIData *data, char *name, char *value, 
+      WsXmlNodeH resource, const char *resource_uri)
 {
   debug("xml2property([0x%04x]%s:%s)", data->type, name, value);
-        xml2data(data, value);
+        xml2data(data, value, name, resource, resource_uri);
         CMSetProperty(instance, name, &(data->value), data->type);
 }
 
@@ -540,7 +567,7 @@ comparef(const void *key1, const void *key2)
 }
 
 
-static void
+static int
 cim_add_args(CimClientInfo * client, CMPIObjectPath *op,
 		CMPIArgs * argsin)
 {
@@ -548,6 +575,7 @@ cim_add_args(CimClientInfo * client, CMPIObjectPath *op,
 	hnode_t *hn;
 	hash_scan_begin(&hs, client->method_args);
 	list_t *arglist = NULL;
+	int res = 0;
 
 	debug("cim_add_args:");
 	while ((hn = hash_scan_next(&hs))) {
@@ -563,7 +591,7 @@ cim_add_args(CimClientInfo * client, CMPIObjectPath *op,
 		// sort the list first then tag arrays
 		debug("cim_add_args: list count: %u", listcount);
 		if (0 >= listcount) {
-			return; // nothing to do so return
+			return res; // nothing to do so return
 		}
 		list_sort(arglist, comparef);
 		argnode = list_first(arglist);
@@ -588,9 +616,25 @@ cim_add_args(CimClientInfo * client, CMPIObjectPath *op,
 			sentry = (selector_entry *)node_val->data;
 			if (0 < node_val->arraycount) {
 				CMPIArray *arraydata = NULL;
-				int arraytype = sentry->type;
+				lnode_t *t_anode = argnode;
+				int arraytype = sentry->type, kk;
 				node_val->arraycount++; // since value of 0 is singleton and value of 1 is actually 2 count in an array
 				debug("cim_add_args: array key: %s type: %u count: %u", node_val->key, arraytype, node_val->arraycount);
+
+				// first verify if array type is the same for all elements in the array.
+				for (kk = 1; kk < node_val->arraycount; kk++) {
+					t_anode = list_next(arglist, t_anode);
+					if (NULL != t_anode) {
+						selector_entry *t_sentry = (selector_entry *)((methodarglist_t *)t_anode->list_data)->data;
+						char *key = ((methodarglist_t *)t_anode->list_data)->key;
+						debug(" %s[0] = %d, %s[%d] = %d", key, sentry->type, key, kk, t_sentry->type);	
+						if (sentry->type != t_sentry->type) {
+							res = 1;
+							goto error;
+						}
+					}
+				}
+
 				if (0 != arraytype)
 					arraydata = native_new_CMPIArray(node_val->arraycount, CMPI_ref, NULL);
 				else
@@ -599,6 +643,10 @@ cim_add_args(CimClientInfo * client, CMPIObjectPath *op,
 					debug("cim_add_args: array %u object: %p", jj, sentry);
 					if (0 != arraytype) {
 						value.ref = cim_epr_to_objectpath(client, sentry->entry.eprp);
+						if (value.ref == NULL) {
+						        res = 1;
+							goto error;
+						}
 						CMSetArrayElementAt(arraydata, jj, &value, CMPI_ref);
 					} else {
 						value.string = native_new_CMPIString((char *)sentry->entry.text, NULL);
@@ -618,6 +666,10 @@ cim_add_args(CimClientInfo * client, CMPIObjectPath *op,
 				debug("cim_add_args: single key: %s type: %u", node_val->key, sentry->type);
 				if (0 != sentry->type) {
 					epr_t *eprp = sentry->entry.eprp;
+					if (eprp == NULL) {
+						res = 1;
+						break;
+					}
 					debug("epr_t: selectorcount: %u", eprp->refparams.selectorset.count);
 					value.ref = cim_epr_to_objectpath(client, sentry->entry.eprp);
 					CMAddArg(argsin, node_val->key, &value, CMPI_ref);
@@ -631,8 +683,12 @@ cim_add_args(CimClientInfo * client, CMPIObjectPath *op,
 	} else {
 		debug("cim_add_args: did not find any argument list");
 	}
+
+error:
+	return res;
 }
 
+
 static void
 cim_add_keys(CMPIObjectPath * objectpath, hash_t * keys)
 {
@@ -763,9 +819,6 @@ cim_verify_keys(CMPIObjectPath * objectpath, hash_t * keys,
 		count = (int) hash_count(keys);
 	}
 	opcount = CMGetKeyCount(objectpath, &rc);
-	debug("getKeyCount rc=%d, msg=%s",
-			rc.rc, (rc.msg) ? CMGetCharPtr(rc.msg) : NULL);
-
 
 	debug("selector count from user: %d, in object path: %d", count,
 			opcount);
@@ -787,7 +840,7 @@ cim_verify_keys(CMPIObjectPath * objectpath, hash_t * keys,
 		if (rc.rc != 0) {	// key not found
 			statusP->fault_code = WSMAN_INVALID_SELECTORS;
 			statusP->fault_detail_code = WSMAN_DETAIL_UNEXPECTED_SELECTORS;
-			debug("unexpcted selectors");
+			debug("unexpected selectors");
 			break;
 		}
 		selector_entry *sentry = (selector_entry*)hnode_get(hn);
@@ -802,7 +855,7 @@ cim_verify_keys(CMPIObjectPath * objectpath, hash_t * keys,
 				statusP->fault_code = WSA_DESTINATION_UNREACHABLE;
 				statusP->fault_detail_code = WSMAN_DETAIL_INVALID_RESOURCEURI;
 				debug("selector '%s', value: [ %s ] not matched", hnode_getkey(hn), sentry->entry.text);
-			        debug("data.type 0x%x, cv '%s'", data.type, cv?cv:"<NULL>");
+			        debug("data.type 0x%04x, cv '%s'", data.type, cv?cv:"<NULL>");
 				u_free(cv);
 				break;
 			}
@@ -978,6 +1031,13 @@ instance2xml(CimClientInfo * client,
 
 
 
+/*
+ * An operation (for a concrete instance) is given only the abstract base class
+ * 
+ * Enumerate through all instances of the base class, comparing keys to identify
+ * the matching instance
+ *
+ */
 
 static CMPIObjectPath *
 cim_get_op_from_enum(CimClientInfo * client,
@@ -1122,6 +1182,10 @@ cim_enum_instances(CimClientInfo * client,
 		enumeration = cc->ft->execQuery(cc, objectpath, filter->query, "WQL", &rc);
 	} else if (( enumInfo->flags & WSMAN_ENUMINFO_CQL )) {
 		enumeration = cc->ft->execQuery(cc, objectpath, filter->query, "CQL", &rc);
+	} else if (( enumInfo->flags & WSMAN_ENUMINFO_XPATH )) { /* XPath unsupported in Sfcc */
+                status->fault_code = WSEN_CANNOT_PROCESS_FILTER;
+                status->fault_detail_code = WSMAN_DETAIL_NOT_SUPPORTED;
+                goto cleanup;
 	} else {
 		enumeration = cc->ft->enumInstances(cc, objectpath,
 				CMPI_FLAG_DeepInheritance,
@@ -1151,6 +1215,7 @@ cim_enum_instances(CimClientInfo * client,
 			CMPIData d = enumArr->ft->getElementAt(enumArr, idx, NULL);
 			if (filter_instance(d.value.inst, enumInfo)) {
 				fenumArr->ft->setElementAt(fenumArr, fidx, &d.value, d.type);
+				fidx++;
 			}
 		}
 	} else {
@@ -1178,7 +1243,13 @@ cleanup:
 	return;
 }
 
-int
+
+/*
+ * Get enumeration item as element at enumInfo->index and append it to itemsNode
+ * return 1 on success
+ */
+
+static int
 cim_getElementAt(CimClientInfo * client,
 		WsEnumerateInfo * enumInfo, WsXmlNodeH itemsNode)
 {
@@ -1193,7 +1264,7 @@ cim_getElementAt(CimClientInfo * client,
 	CMPIObjectPath *objectpath = instance->ft->getObjectPath(instance, NULL);
 	CMPIString *classname = objectpath->ft->getClassName(objectpath, NULL);
 
-	if (enumInfo && (enumInfo->flags & WSMAN_ENUMINFO_POLY_NONE ) &&
+	if ((enumInfo->flags & WSMAN_ENUMINFO_POLY_NONE ) &&
 			(strcmp(CMGetCharPtr(classname), client->requested_class) != 0)) {
 		retval = 0;
 	}
@@ -1213,14 +1284,16 @@ cim_getElementAt(CimClientInfo * client,
 }
 
 
+/*
+ * Get enumeration item as WS end point reference at enumInfo->index and append it to itemsNode
+ * return 1 on success
+ * 
+ */
 
-
-
-int
+static int
 cim_getEprAt(CimClientInfo * client,
 		WsEnumerateInfo * enumInfo, WsXmlNodeH itemsNode)
 {
-
 	int retval = 1;
 	char *uri = NULL;
 	CMPIArray *results = (CMPIArray *) enumInfo->enumResults;
@@ -1231,7 +1304,7 @@ cim_getEprAt(CimClientInfo * client,
 	CMPIObjectPath *objectpath = instance->ft->getObjectPath(instance, NULL);
 	CMPIString *classname = objectpath->ft->getClassName(objectpath, NULL);
 
-	if (enumInfo && (enumInfo->flags & WSMAN_ENUMINFO_POLY_NONE)
+	if ((enumInfo->flags & WSMAN_ENUMINFO_POLY_NONE)
 			&& (strcmp(CMGetCharPtr(classname), client->requested_class) != 0)) {
 		retval = 0;
 	}
@@ -1248,7 +1321,14 @@ cim_getEprAt(CimClientInfo * client,
 	return retval;
 }
 
-int
+
+/*
+ * Get enumeration item as WS end point reference object at enumInfo->index and append it to itemsNode
+ * return 1 on success
+ * 
+ */
+
+static int
 cim_getEprObjAt(CimClientInfo * client,
 		WsEnumerateInfo * enumInfo, WsXmlNodeH itemsNode)
 {
@@ -1264,7 +1344,7 @@ cim_getEprObjAt(CimClientInfo * client,
 	CMPIString *classname =
 		objectpath->ft->getClassName(objectpath, NULL);
 
-	if (enumInfo && (enumInfo->flags & WSMAN_ENUMINFO_POLY_NONE) &&
+	if ((enumInfo->flags & WSMAN_ENUMINFO_POLY_NONE) &&
 			(strcmp(CMGetCharPtr(classname), client->requested_class) != 0)) {
 		retval = 0;
 	}
@@ -1272,8 +1352,8 @@ cim_getEprObjAt(CimClientInfo * client,
 
 	if (retval) {
 		WsXmlNodeH item =
-			ws_xml_add_child(itemsNode, XML_NS_WS_MAN, WSM_ITEM,
-					NULL);
+                        ws_xml_add_child(itemsNode, XML_NS_WS_MAN, WSM_ITEM,
+                                        NULL);
 		instance2xml(client, instance, NULL, item, enumInfo);
 		cim_add_epr(client, item, uri, objectpath);
 	}
@@ -1331,7 +1411,7 @@ create_instance_from_xml(CMPIInstance * instance,
 				if (value) {
 					xml2property(instance, &data,
 							CMGetCharPtr(propertyname),
-							value);
+							value, resource, resource_uri);
 				}
 			} else if (data.type != CMPI_null
 					&& data.state != CMPI_nullValue) {
@@ -1375,7 +1455,7 @@ create_instance_from_xml(CMPIInstance * instance,
 			debug("prop value: %s", value );
 			if (value) {
 				xml2property(instance, &data,
-						element, value);
+						element, value, resource, NULL);
 			}
 		}
 	}
@@ -1423,7 +1503,7 @@ xml2instance(CMPIInstance * instance, WsXmlNodeH body, char *resourceUri)
 			if (value) {
 				xml2property(instance, &data,
 						CMGetCharPtr(propertyname),
-						value);
+						value, body, resourceUri);
 			}
 			CMRelease(propertyname);
 		}
@@ -1535,15 +1615,24 @@ cim_connect_to_cimom(char *cim_host,
 {
 	CMPIStatus rc;
         memset(&rc, 0, sizeof(CMPIStatus)); /* workaround for sfcb bug #2844812 */
-	if (strcmp(frontend, "SfcbLocal") != 0)
-		frontend = "http";
+	if (strcmp(frontend, "SfcbLocal") != 0) {
+          if (get_cim_ssl())
+            frontend = "https";
+          else
+            frontend = "http";
+        }
 
-	CMCIClient *cimclient = cmciConnect(cim_host, frontend , cim_port,
+	CMCIClient *cimclient = cmciConnect2(cim_host, frontend, cim_port,
 			cim_host_userid,
-			cim_host_passwd, &rc);
+			cim_host_passwd,
+                        get_cim_verify(),
+                        get_cim_trust_store(),
+                        NULL, /* certFile */
+                        NULL, /* keyFile */
+                        &rc);
 
 	if (cimclient == NULL) {
-	        debug( "*** Connection to CIMOM %s://%s:%s failed with %d:%s", frontend, cim_host, cim_port, rc.rc, CMGetCharPtr(rc.msg));
+	        debug( "*** Connection to CIMOM %s://%s:%s failed with %d:%s", frontend, cim_host, cim_port, rc.rc, rc.msg ? CMGetCharPtr(rc.msg) : "NewCIMCEnv failed in sfcc");
 	} else {
 		debug("new cimclient: 0x%8x", cimclient);
 		debug("new cimclient: %d", cimclient->ft->ftVersion);
@@ -1565,7 +1654,7 @@ release_cmpi_data(CMPIData data)
 {
         if (data.state == CMPI_nullValue)
                 return;
-        debug("release_cmpi_data, type = %d",data.type);
+        debug("release_cmpi_data, type = 0x%04x", data.type);
         switch(data.type)
         {
                 case CMPI_string:
@@ -1604,8 +1693,11 @@ invoke_enumerate_class_names(CimClientInfo *client, WsXmlNodeH body, CMPIStatus
 {
 	CMPIObjectPath *op = newCMPIObjectPath(client->cim_namespace, "", NULL);
 	CMCIClient *cc = (CMCIClient *)client->cc;
-	CMPIEnumeration *classnames = cc->ft->enumClassNames(cc, op, client->flags | CMPI_FLAG_DeepInheritance, rc);
-  
+        unsigned long flags = client->flags;
+        if (client->selectors && hash_lookup(client->selectors, (char *) "DeepInheritance"))
+                flags |= CMPI_FLAG_DeepInheritance;
+	CMPIEnumeration *classnames = cc->ft->enumClassNames(cc, op, flags, rc);
+
         debug("invoke_enumerate_class_names");
   
 	if (classnames) {
@@ -1686,10 +1778,12 @@ cim_invoke_method(CimClientInfo * client,
 
 	if (client->resource_uri
 	    && strstr(client->resource_uri, XML_NS_CIM_CLASS) != NULL) {
-	        /* uri specifies class */
+	        /* Generic CIM uri given (pointing to abstract CIM_xxx class
+		 * enumerate through all instances of this class to
+		 * get the correct instance matching the selectors
+		 */
 		objectpath = cim_get_op_from_enum(client, status);
 	} else {
-		debug("no base class, getting instance directly with getInstance");
 		objectpath = newCMPIObjectPath(client->cim_namespace,
 				client->requested_class, NULL);
 		if (objectpath != NULL)
@@ -1708,7 +1802,11 @@ cim_invoke_method(CimClientInfo * client,
 
 		if (client->method_args && hash_count(client->method_args) > 0) {
 			debug("adding method arguments");
-			cim_add_args(client, objectpath, argsin);
+			if (0 != cim_add_args(client, objectpath, argsin)) {
+				status->fault_code = WSMAN_INVALID_PARAMETER;
+				status->fault_detail_code = WSMAN_DETAIL_INVALID_VALUE;
+				goto cleanup;
+			}
 		}
 
 	        if (strstr(client->resource_uri, XML_NS_CIM_INTRINSIC) != NULL) {
@@ -1758,6 +1856,7 @@ cim_invoke_method(CimClientInfo * client,
 		cim_to_wsman_status(rc, status);
 		if (rc.msg)
 			CMRelease(rc.msg);
+cleanup:
 		if (argsin)
 			CMRelease(argsin);
 		if (argsout)
@@ -1905,15 +2004,15 @@ cim_put_instance(CimClientInfo * client,
 		rc = cc->ft->setInstance(cc, objectpath, instance, 0, NULL);
 		debug("modifyInstance() rc=%d, msg=%s", rc.rc,
 				(rc.msg) ? (char *) CMGetCharPtr(rc.msg) : NULL);
-		if (rc.rc == CMPI_RC_ERR_FAILED) {
-			status->fault_code =
-				WSA_ACTION_NOT_SUPPORTED;
-		} else {
-			cim_to_wsman_status(rc, status);
-		}
-		if (rc.rc == 0) {
+		cim_to_wsman_status(rc, status);
+		if (rc.rc == CMPI_RC_OK) {
+			// return the current representation of the resource
+			instance = cc->ft->getInstance(cc, objectpath,
+					CMPI_FLAG_IncludeClassOrigin,
+					NULL, &rc);
 			instance2xml(client, instance, fragstr, body, NULL);
 		}
+
 		if (rc.msg)
 			CMRelease(rc.msg);
 	}
@@ -2155,6 +2254,46 @@ cleanup:
 
 }
 
+CMPIObjectPath *
+cim_get_indicationfilter_objectpath_from_selectors(CimClientInfo * client,
+        WsContextH cntx, WsmanStatus * status)
+{
+    CMPIObjectPath *objectpath = NULL;
+
+    CMPIConstClass *class = NULL;
+
+    objectpath = newCMPIObjectPath(
+            get_indication_profile_implementation_ns(),
+            client->requested_class, NULL);
+
+    if(objectpath) {
+        CMPIStatus rc;
+        CMCIClient *cc = (CMCIClient *)client->cc;
+        class = cc->ft->getClass(cc,
+                                 objectpath,
+                                 CMPI_FLAG_IncludeQualifiers,
+                                 NULL,
+                                 &rc);
+        if (!class){
+            CMRelease(objectpath);
+            goto cleanup;
+        }
+
+        cim_verify_class_keys(class, client->selectors, status);
+        if (status->fault_code != 0){
+            CMRelease(objectpath);
+            goto cleanup;
+        }
+
+        cim_add_keys(objectpath, client->selectors);
+    }
+cleanup:
+    if (class)
+        CMRelease(class);
+    return objectpath;
+
+}
+
 CMPIObjectPath *
 cim_get_objectpath_from_selectors(CimClientInfo * client,
 		WsContextH cntx, WsmanStatus * status)
@@ -2197,7 +2336,8 @@ static CMPIObjectPath *
 cim_indication_filter_objectpath(CimClientInfo *client,
 		WsSubscribeInfo *subsInfo, CMPIStatus *rc)
 {
-	CMPIObjectPath *objectpath_filter = newCMPIObjectPath(subsInfo->cim_namespace,
+	CMPIObjectPath *objectpath_filter = newCMPIObjectPath(
+            get_indication_profile_implementation_ns(),
 			"CIM_IndicationFilter", rc);
 	CMAddKey(objectpath_filter, "SystemCreationClassName",
 			SYSTEMCREATIONCLASSNAME, CMPI_chars);
@@ -2213,7 +2353,8 @@ cim_indication_filter_objectpath(CimClientInfo *client,
 static CMPIObjectPath
 *cim_indication_handler_objectpath(CimClientInfo *client, WsSubscribeInfo *subsInfo, CMPIStatus *rc)
 {
-	CMPIObjectPath *objectpath_handler = newCMPIObjectPath(subsInfo->cim_namespace,
+	CMPIObjectPath *objectpath_handler = newCMPIObjectPath(
+            get_indication_profile_implementation_ns(),
 			"CIM_IndicationHandlerCIMXML", rc);
 	CMAddKey(objectpath_handler, "SystemCreationClassName",
 			SYSTEMCREATIONCLASSNAME, CMPI_chars);
@@ -2249,7 +2390,7 @@ cim_create_indication_filter(CimClientInfo *client, WsSubscribeInfo *subsInfo, W
 	else if(subsInfo->flags & WSMAN_SUBSCRIPTION_CQL)
 		CMAddKey(objectpath, "QueryLanguage",
 				"CQL",CMPI_chars);
-	char *indicationns = get_cim_indication_SourceNamespace();
+	char *indicationns = subsInfo->cim_namespace;
 	if(indicationns)
 		CMAddKey(objectpath, "SourceNamespace",
 				indicationns, CMPI_chars);
@@ -2354,7 +2495,8 @@ cim_create_indication_subscription(CimClientInfo * client, WsSubscribeInfo *subs
 	//	objectpath_handler = cim_indication_handler_objectpath(client, subsInfo, &rc);
 	//	if(rc.rc) goto cleanup;
 
-	objectpath = newCMPIObjectPath(client->cim_namespace,
+	objectpath = newCMPIObjectPath(
+            get_indication_profile_implementation_ns(),
 			"CIM_IndicationSubscription", NULL);
 	CMPIValue value;
 	value.ref = filter;
@@ -2363,26 +2505,26 @@ cim_create_indication_subscription(CimClientInfo * client, WsSubscribeInfo *subs
 	value.ref = handler;
 	CMAddKey(objectpath, "Handler",
 			&value, CMPI_ref);
+        instance = newCMPIInstance(objectpath, NULL);
 	//set OnFatalErrorPolicy to "Ignore"
 	value.uint16 = 2;
-	CMAddKey(objectpath, "OnFatalErrorPolicy",
+        CMSetProperty(instance, "OnFatalErrorPolicy",
 			&value, CMPI_uint16);
 	//enable subscription
 	value.uint16 = 2;
-	CMAddKey(objectpath, "SubscriptionState",
+        CMSetProperty(instance, "SubscriptionState",
 			&value, CMPI_uint16);
 	if(subsInfo->expires) {
 		struct timeval  tv;
 		gettimeofday(&tv, NULL);
 		value.uint64 = subsInfo->expires - tv.tv_sec;
-		CMAddKey(objectpath, "subscriptionDuration",
+                CMSetProperty(instance, "SubscriptionDuration",
 				&value, CMPI_uint64);
 	}
 	//set RepeatNotificationPolicy to None
 	value.uint16 = 2;
-	CMAddKey(objectpath, "RepeatNotificationPolicy",
+        CMSetProperty(instance, "RepeatNotificationPolicy",
 			&value, CMPI_uint16);
-	instance = newCMPIInstance(objectpath, NULL);
 	instance_r = cc->ft->createInstance(cc, objectpath, instance, &rc);
 
 	/* Print the results */
@@ -2424,7 +2566,8 @@ cim_update_indication_subscription(CimClientInfo *client, WsSubscribeInfo *subsI
 	objectpath_handler = cim_indication_handler_objectpath(client, subsInfo, &rc);
 	if(rc.rc)
 		goto cleanup;
-	objectpath = newCMPIObjectPath(client->cim_namespace,
+	objectpath = newCMPIObjectPath(
+            get_indication_profile_implementation_ns(),
 			"CIM_IndicationSubscription", NULL);
 	CMPIValue value;
 	value.ref = objectpath_filter;
@@ -2480,7 +2623,8 @@ cim_delete_indication_subscription(CimClientInfo *client, WsSubscribeInfo *subsI
 	objectpath_handler = cim_indication_handler_objectpath(client, subsInfo, &rc);
 	if(rc.rc)
 		goto cleanup;
-	objectpath_subscription = newCMPIObjectPath(subsInfo->cim_namespace,
+	objectpath_subscription = newCMPIObjectPath(
+            get_indication_profile_implementation_ns(),
 			"CIM_IndicationSubscription", &rc);
 	if(rc.rc)
 		goto cleanup;
@@ -2706,8 +2850,8 @@ cim_get_enum_items(CimClientInfo * client,
 {
 	WsXmlNodeH itemsNode;
 	WsXmlDocH outdoc = NULL;
-	WsXmlDocH clonedoc = NULL;
-	int c = 0;
+        int c;
+        int count = 0;
 	if (node == NULL)
 		return;
 
@@ -2715,67 +2859,46 @@ cim_get_enum_items(CimClientInfo * client,
 	debug("Total items: %d", enumInfo->totalItems);
 	debug("enum flags: %lu", enumInfo->flags );
 
-	if(enumInfo->totalItems == 0) {
-		outdoc = ws_xml_get_node_doc(node);
-		enumInfo->pullResultPtr = outdoc;
-	} else if (maxelements> 0) {
-		while (maxelements> 0 && enumInfo->index >= 0 &&
+	outdoc = ws_xml_get_node_doc(node);
+	if (enumInfo->totalItems > 0) {
+                if (maxelements <= 0) {
+                        maxelements = -1; /* don't check maxelements */
+                }
+		while (enumInfo->index >= 0 &&
 				enumInfo->index < enumInfo->totalItems) {
-			outdoc = ws_xml_get_node_doc(node);
-			ws_xml_destroy_doc(clonedoc);
-			clonedoc = ws_xml_duplicate_doc(outdoc);
 			if (enumInfo->flags & WSMAN_ENUMINFO_EPR ) {
-				c = cim_getEprAt(client, enumInfo,
-						itemsNode);
+				c = cim_getEprAt(client, enumInfo, itemsNode);
 			} else if (enumInfo->flags & WSMAN_ENUMINFO_OBJEPR) {
-				c = cim_getEprObjAt(client, enumInfo,
-						itemsNode);
-			} else {
-				c = cim_getElementAt(client, enumInfo,
-						itemsNode);
-			}
-			if(check_envelope_size(outdoc, maxsize, enumInfo->encoding)) {
-				enumInfo->pullResultPtr = clonedoc;
-				clonedoc = NULL;
-				ws_xml_destroy_doc(outdoc);
-				break;
-			}
-			enumInfo->index++;
-			maxelements--;
-		}
-		enumInfo->index--;
-		if(clonedoc) {
-			enumInfo->pullResultPtr = outdoc;
-		}
-		ws_xml_destroy_doc(clonedoc);
-	} else {
-		while (enumInfo->index >= 0
-				&& enumInfo->index < enumInfo->totalItems) {
-			outdoc = ws_xml_get_node_doc(node);
-			ws_xml_destroy_doc(clonedoc);
-			clonedoc = ws_xml_duplicate_doc(outdoc);
-			if (enumInfo->flags & WSMAN_ENUMINFO_EPR ) {
-				c = cim_getEprAt(client, enumInfo,
-						itemsNode);
-			} else if (enumInfo->flags & WSMAN_ENUMINFO_OBJEPR) {
-				c = cim_getEprObjAt(client, enumInfo,
-						itemsNode);
+				c = cim_getEprObjAt(client, enumInfo, itemsNode);
 			} else {
 				c = cim_getElementAt(client, enumInfo, itemsNode);
 			}
-			if(check_envelope_size(outdoc, enumInfo->maxsize, enumInfo->encoding)) {
-				enumInfo->pullResultPtr = clonedoc;
-				clonedoc = NULL;
-				ws_xml_destroy_doc(outdoc);
+                        if (!c) {
+                                /* cim_getE... failed */
+                                break;
+                        }
+			if (check_envelope_size(outdoc, maxsize, enumInfo->encoding)) {
+                                /* last item added to itemsNode exceeded the envelope size */
+                                if (count > 0) {
+                                        /* if there's already a partial result,
+                                         * remove last child from itemsNode
+                                         * and return partial result */
+                                        WsXmlNodeH item = xml_parser_node_get(itemsNode, XML_LAST_CHILD);
+                                        xml_parser_node_remove(item);
+                                }
+                                /* if the first item already exceeds the envelope size, leave it
+                                 * and let the SOAP report an EncodingLimit fault.
+                                 */
 				break;
 			}
 			enumInfo->index++;
+                        count++;
+			maxelements--;
+                        if (maxelements == 0) {
+                                break;
+                        }
 		}
-		enumInfo->index--;
-
-		if(clonedoc) {
-			enumInfo->pullResultPtr = outdoc;
-		}
-		ws_xml_destroy_doc(clonedoc);
+		enumInfo->index--; /* callee (wsman-soap.c) increments it again */
 	}
+	enumInfo->pullResultPtr = outdoc;
 }
